@@ -10,11 +10,11 @@
 use super::{
     files::{FileItem, FileMeta, FilesMap, RealPath},
     multimap::MultimapKeyValues,
-    nrs::NrsMap,
+    nrs::{NrsMap, VersionHash},
     register::{Entry, EntryHash},
     Safe, XorName,
 };
-pub use super::{SafeContentType, SafeDataType, SafeUrl, XorUrlBase};
+pub use super::{ContentType, DataType, NativeUrl, XorUrlBase};
 use crate::{Error, Result};
 use log::{debug, info};
 use serde::{Deserialize, Serialize};
@@ -36,9 +36,9 @@ pub enum SafeData {
         xorurl: String,
         xorname: XorName,
         type_tag: u64,
-        version: u64,
+        version: VersionHash,
         files_map: FilesMap,
-        data_type: SafeDataType,
+        data_type: DataType,
         resolved_from: String,
     },
     PublicBlob {
@@ -54,9 +54,9 @@ pub enum SafeData {
         xorurl: String,
         xorname: XorName,
         type_tag: u64,
-        version: u64,
+        version: VersionHash,
         nrs_map: NrsMap,
-        data_type: SafeDataType,
+        data_type: DataType,
         resolved_from: String,
     },
     Multimap {
@@ -66,22 +66,6 @@ pub enum SafeData {
         data: MultimapKeyValues,
         resolved_from: String,
     },
-    PublicSequence {
-        xorurl: String,
-        xorname: XorName,
-        type_tag: u64,
-        version: u64,
-        data: Vec<u8>,
-        resolved_from: String,
-    },
-    PrivateSequence {
-        xorurl: String,
-        xorname: XorName,
-        type_tag: u64,
-        version: u64,
-        data: Vec<u8>,
-        resolved_from: String,
-    },
     PublicRegister {
         xorurl: String,
         xorname: XorName,
@@ -107,8 +91,6 @@ impl SafeData {
             | PublicBlob { xorurl, .. }
             | NrsMapContainer { xorurl, .. }
             | Multimap { xorurl, .. }
-            | PublicSequence { xorurl, .. }
-            | PrivateSequence { xorurl, .. }
             | PublicRegister { xorurl, .. }
             | PrivateRegister { xorurl, .. } => xorurl.clone(),
         }
@@ -122,8 +104,6 @@ impl SafeData {
             | PublicBlob { resolved_from, .. }
             | NrsMapContainer { resolved_from, .. }
             | Multimap { resolved_from, .. }
-            | PrivateSequence { resolved_from, .. }
-            | PublicSequence { resolved_from, .. }
             | PublicRegister { resolved_from, .. }
             | PrivateRegister { resolved_from, .. } => resolved_from.clone(),
         }
@@ -263,7 +243,7 @@ impl Safe {
 
     async fn resolve_one_indirection(
         &self,
-        mut the_xor: SafeUrl,
+        mut the_xor: NativeUrl,
         metadata: Option<FileItem>,
         retrieve_data: bool,
         range: Range,
@@ -273,7 +253,7 @@ impl Safe {
         let xorurl = the_xor.to_xorurl_string();
         debug!("Going into a new step in the URL resolution for {}, content type: {:?}, data type: {:?}", xorurl, the_xor.content_type(), the_xor.data_type());
         match the_xor.content_type() {
-            SafeContentType::FilesContainer => {
+            ContentType::FilesContainer => {
                 if !the_xor.sub_names_vec().is_empty() {
                     let msg = format!(
                         "Cannot resolve FilesContainer URL as it contains subnames: {}",
@@ -301,7 +281,7 @@ impl Safe {
                                 if FileMeta::filetype_is_file(file_type) {
                                     match file_item.get("link") {
                                         Some(link) => {
-                                            let new_target_xorurl = SafeUrl::from_url(link)?;
+                                            let new_target_xorurl = NativeUrl::from_url(link)?;
                                             let mut metadata = (*file_item).clone();
                                             Path::new(&path).file_name().map(|name| {
                                                 name.to_str().map(|str| {
@@ -352,7 +332,7 @@ impl Safe {
 
                 Ok((safe_data, next))
             }
-            SafeContentType::NrsMapContainer => {
+            ContentType::NrsMapContainer => {
                 let (version, nrs_map) = self
                     .nrs_map_container_get(&xorurl)
                     .await
@@ -404,7 +384,7 @@ impl Safe {
 
                 Ok((nrs_map_container, Some((target_safe_url, None))))
             }
-            SafeContentType::Multimap => {
+            ContentType::Multimap => {
                 let data = if retrieve_data {
                     // TODO: pass the hash of the URL to grab a single element
                     self.fetch_multimap_values(&the_xor).await?
@@ -422,7 +402,7 @@ impl Safe {
 
                 Ok((safe_data, None))
             }
-            SafeContentType::Raw => {
+            ContentType::Raw => {
                 if !the_xor.sub_names_vec().is_empty() {
                     let msg = format!(
                         "Cannot resolve URL targetting raw content as it contains subnames: {}",
@@ -433,7 +413,7 @@ impl Safe {
                 }
 
                 match the_xor.data_type() {
-                    SafeDataType::SafeKey => {
+                    DataType::SafeKey => {
                         let safe_data = SafeData::SafeKey {
                             xorurl,
                             xorname: the_xor.xorname(),
@@ -441,43 +421,11 @@ impl Safe {
                         };
                         Ok((safe_data, None))
                     }
-                    SafeDataType::PublicBlob => {
+                    DataType::Blob => {
                         self.retrieve_blob(&the_xor, retrieve_data, None, &metadata, range)
                             .await
                     }
-                    SafeDataType::PublicSequence => {
-                        // TODO: fetch only if 'retrieve_data' is set,
-                        // although we need the version regardless
-                        let (version, data) = self.fetch_sequence(&the_xor).await?;
-                        debug!("Data found with v:{}, on Sequence at: {}", version, xorurl);
-                        let safe_data = SafeData::PublicSequence {
-                            xorurl,
-                            xorname: the_xor.xorname(),
-                            type_tag: the_xor.type_tag(),
-                            version,
-                            data: if retrieve_data { data } else { vec![] },
-                            resolved_from: url.to_string(),
-                        };
-
-                        Ok((safe_data, None))
-                    }
-                    SafeDataType::PrivateSequence => {
-                        // TODO: fetch only if 'retrieve_data' is set,
-                        // although we need the version regardless
-                        let (version, data) = self.fetch_sequence(&the_xor).await?;
-                        debug!("Data found with v:{}, on Sequence at: {}", version, xorurl);
-                        let safe_data = SafeData::PrivateSequence {
-                            xorurl,
-                            xorname: the_xor.xorname(),
-                            type_tag: the_xor.type_tag(),
-                            version,
-                            data: if retrieve_data { data } else { vec![] },
-                            resolved_from: url.to_string(),
-                        };
-
-                        Ok((safe_data, None))
-                    }
-                    SafeDataType::PublicRegister => {
+                    DataType::Register => {
                         let data = if retrieve_data {
                             // TODO: use the content hash in the URL to grab a single element if it exists
                             self.fetch_register_entries(&the_xor).await?
@@ -495,31 +443,9 @@ impl Safe {
 
                         Ok((safe_data, None))
                     }
-                    SafeDataType::PrivateRegister => {
-                        let data = if retrieve_data {
-                            // TODO: use the content hash in the URL to grab a single element if it exists
-                            self.fetch_register_entries(&the_xor).await?
-                        } else {
-                            BTreeSet::new()
-                        };
-
-                        let safe_data = SafeData::PrivateRegister {
-                            xorurl,
-                            xorname: the_xor.xorname(),
-                            type_tag: the_xor.type_tag(),
-                            data,
-                            resolved_from: url.to_string(),
-                        };
-
-                        Ok((safe_data, None))
-                    }
-                    other => Err(Error::ContentError(format!(
-                        "Data type '{:?}' not supported yet",
-                        other
-                    ))),
                 }
             }
-            SafeContentType::MediaType(media_type_str) => {
+            ContentType::MediaType(media_type_str) => {
                 if !the_xor.sub_names_vec().is_empty() {
                     let msg = format!(
                         "Cannot resolve URL targetting raw content as it contains subnames: {}",
@@ -530,7 +456,7 @@ impl Safe {
                 }
 
                 match the_xor.data_type() {
-                    SafeDataType::PublicBlob => {
+                    DataType::Blob => {
                         self.retrieve_blob(
                             &the_xor,
                             retrieve_data,
@@ -546,7 +472,7 @@ impl Safe {
                     ))),
                 }
             }
-            SafeContentType::Wallet { .. } => Err(Error::EmptyContent(
+            ContentType::Wallet { .. } => Err(Error::EmptyContent(
                 "Temporarily disabled feature".to_string(),
             )),
         }
@@ -554,7 +480,7 @@ impl Safe {
 
     async fn retrieve_blob(
         &self,
-        the_xor: &SafeUrl,
+        the_xor: &NativeUrl,
         retrieve_data: bool,
         media_type: Option<String>,
         metadata: &Option<FileItem>,
@@ -614,12 +540,12 @@ fn gen_filtered_filesmap(urlpath: &str, files_map: &FilesMap, xorurl: &str) -> R
 }
 // // This contains information for the next step to be made
 // // in each iteration of the resolution process
-type NextStepInfo = (SafeUrl, Option<FileItem>);
+type NextStepInfo = (NativeUrl, Option<FileItem>);
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{app::test_helpers::new_safe_instance, retry_loop, SafeUrl};
+    use crate::{app::test_helpers::new_safe_instance, retry_loop, NativeUrl, Scope};
     use anyhow::{anyhow, bail, Context, Result};
     use rand::{distributions::Alphanumeric, thread_rng, Rng};
     use std::io::Read;
@@ -631,8 +557,9 @@ mod tests {
             .files_container_create(Some("../testdata/"), None, true, false, false)
             .await?;
 
-        let safe_url = SafeUrl::from_url(&xorurl)?;
+        let safe_url = NativeUrl::from_url(&xorurl)?;
         let content = retry_loop!(safe.fetch(&xorurl, None));
+        let (version0, _) = retry_loop!(safe.files_container_get(&xorurl));
 
         assert!(
             content
@@ -640,9 +567,9 @@ mod tests {
                     xorurl: xorurl.clone(),
                     xorname: safe_url.xorname(),
                     type_tag: 1_100,
-                    version: 0,
+                    version: version0,
                     files_map,
-                    data_type: SafeDataType::PublicSequence,
+                    data_type: DataType::Register,
                     resolved_from: xorurl.clone(),
                 }
         );
@@ -672,9 +599,10 @@ mod tests {
             .files_container_create(Some("../testdata/"), None, true, false, false)
             .await?;
         let _ = retry_loop!(safe.fetch(&xorurl, None));
+        let (version0, _) = retry_loop!(safe.files_container_get(&xorurl));
 
-        let mut safe_url = SafeUrl::from_url(&xorurl)?;
-        safe_url.set_content_version(Some(0));
+        let mut safe_url = NativeUrl::from_url(&xorurl)?;
+        safe_url.set_content_version(Some(version0));
         let (_nrs_map_xorurl, _, _nrs_map) = safe
             .nrs_map_container_create(&site_name, &safe_url.to_string(), true, true, false)
             .await?;
@@ -699,8 +627,8 @@ mod tests {
                 assert_eq!(*xorurl, xorurl_without_subname);
                 assert_eq!(*xorname, safe_url.xorname());
                 assert_eq!(*type_tag, 1_100);
-                assert_eq!(*version, 0);
-                assert_eq!(*data_type, SafeDataType::PublicSequence);
+                assert_eq!(*version, version0);
+                assert_eq!(*data_type, DataType::Register);
                 assert_eq!(*files_map, the_files_map);
 
                 // let's also compare it with the result from inspecting the URL
@@ -722,9 +650,10 @@ mod tests {
             .files_container_create(Some("../testdata/"), None, true, false, false)
             .await?;
         let _ = retry_loop!(safe.fetch(&xorurl, None));
+        let (version0, _) = retry_loop!(safe.files_container_get(&xorurl));
 
-        let mut safe_url = SafeUrl::from_url(&xorurl)?;
-        safe_url.set_content_version(Some(0));
+        let mut safe_url = NativeUrl::from_url(&xorurl)?;
+        safe_url.set_content_version(Some(version0));
         let files_container_url = safe_url.to_string();
         let _ = safe
             .nrs_map_container_create(&site_name, &files_container_url, true, true, false)
@@ -761,7 +690,7 @@ mod tests {
             .files_store_public_blob(data, Some("text/plain"), false)
             .await?;
 
-        let safe_url = SafeUrl::from_url(&xorurl)?;
+        let safe_url = NativeUrl::from_url(&xorurl)?;
         let content = retry_loop!(safe.fetch(&xorurl, None));
         assert!(
             content
@@ -835,9 +764,10 @@ mod tests {
             .files_container_create(Some("../testdata/"), None, true, false, false)
             .await?;
         let _ = retry_loop!(safe.fetch(&xorurl, None));
+        let (version0, _) = retry_loop!(safe.files_container_get(&xorurl));
 
-        let mut safe_url = SafeUrl::from_url(&xorurl)?;
-        safe_url.set_content_version(Some(0));
+        let mut safe_url = NativeUrl::from_url(&xorurl)?;
+        safe_url.set_content_version(Some(version0));
         let (_nrs_map_xorurl, _, _nrs_map) = safe
             .nrs_map_container_create(&site_name, &safe_url.to_string(), true, true, false)
             .await?;
@@ -884,95 +814,18 @@ mod tests {
         }
     }
 
-    #[tokio::test]
-    async fn test_fetch_public_sequence() -> Result<()> {
-        let mut safe = new_safe_instance().await?;
-        let data = b"Something super immutable";
-        let xorurl = safe.sequence_create(data, None, 25_000, false).await?;
-
-        let safe_url = SafeUrl::from_url(&xorurl)?;
-        let content = retry_loop!(safe.fetch(&xorurl, None));
-        assert!(
-            content
-                == SafeData::PublicSequence {
-                    xorurl: xorurl.clone(),
-                    xorname: safe_url.xorname(),
-                    type_tag: safe_url.type_tag(),
-                    version: 0,
-                    data: data.to_vec(),
-                    resolved_from: xorurl.clone(),
-                }
-        );
-
-        // let's also compare it with the result from inspecting the URL
-        let inspected_url = safe.inspect(&xorurl).await?;
-        assert!(
-            inspected_url[0]
-                == SafeData::PublicSequence {
-                    xorurl: xorurl.clone(),
-                    xorname: safe_url.xorname(),
-                    type_tag: safe_url.type_tag(),
-                    version: 0,
-                    data: vec![],
-                    resolved_from: xorurl,
-                }
-        );
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn test_fetch_unsupported() -> Result<()> {
-        let mut safe = new_safe_instance().await?;
-        let xorname = rand::random();
-        let type_tag = 575_756_443;
-        let xorurl = SafeUrl::encode(
-            xorname,
-            None,
-            type_tag,
-            SafeDataType::PrivateBlob,
-            SafeContentType::Raw,
-            None,
-            None,
-            None,
-            None,
-            None,
-            XorUrlBase::Base32z,
-        )?;
-
-        match safe.fetch(&xorurl, None).await {
-            Ok(c) => {
-                bail!("Unxpected fetched content: {:?}", c)
-            }
-            Err(Error::ContentError(msg)) => {
-                assert_eq!(msg, "Data type 'PrivateBlob' not supported yet".to_string())
-            }
-            other => bail!("Error returned is not the expected one: {:?}", other),
-        };
-
-        match safe.inspect(&xorurl).await {
-            Ok(c) => Err(anyhow!("Unxpected fetched content: {:?}", c)),
-            Err(Error::ContentError(msg)) => {
-                assert_eq!(msg, "Data type 'PrivateBlob' not supported yet".to_string());
-                Ok(())
-            }
-            other => Err(anyhow!(
-                "Error returned is not the expected one: {:?}",
-                other
-            )),
-        }
-    }
-
     #[tokio::test]
     async fn test_fetch_unsupported_with_media_type() -> Result<()> {
         let mut safe = new_safe_instance().await?;
         let xorname = rand::random();
         let type_tag = 575_756_443;
-        let xorurl = SafeUrl::encode(
+        let xorurl = NativeUrl::encode(
             xorname,
             None,
             type_tag,
-            SafeDataType::PrivateBlob,
-            SafeContentType::MediaType("text/html".to_string()),
+            Scope::Private,
+            DataType::Register,
+            ContentType::MediaType("text/html".to_string()),
             None,
             None,
             None,
@@ -986,7 +839,7 @@ mod tests {
                 bail!("Unxpected fetched content: {:?}", c)
             }
             Err(Error::ContentError(msg)) => {
-                assert_eq!(msg, "Data type 'PrivateBlob' not supported yet".to_string())
+                assert_eq!(msg, "Data type 'Register' not supported yet".to_string())
             }
             other => bail!("Error returned is not the expected one: {:?}", other),
         };
@@ -994,7 +847,7 @@ mod tests {
         match safe.inspect(&xorurl).await {
             Ok(c) => Err(anyhow!("Unxpected fetched content: {:?}", c)),
             Err(Error::ContentError(msg)) => {
-                assert_eq!(msg, "Data type 'PrivateBlob' not supported yet".to_string());
+                assert_eq!(msg, "Data type 'Register' not supported yet".to_string());
                 Ok(())
             }
             other => Err(anyhow!(
@@ -1010,7 +863,7 @@ mod tests {
         let data = b"Something super immutable";
         let xorurl = safe.files_store_public_blob(data, None, false).await?;
 
-        let mut safe_url = SafeUrl::from_url(&xorurl)?;
+        let mut safe_url = NativeUrl::from_url(&xorurl)?;
         let path = "/some_relative_filepath";
         safe_url.set_path(path);
         match safe.fetch(&safe_url.to_string(), None).await {
@@ -1029,7 +882,7 @@ mod tests {
             .files_store_public_blob(data, Some("text/plain"), false)
             .await?;
 
-        let mut safe_url = SafeUrl::from_url(&xorurl)?;
+        let mut safe_url = NativeUrl::from_url(&xorurl)?;
         safe_url.set_path("/some_relative_filepath");
         let url_with_path = safe_url.to_string();
         match safe.fetch(&url_with_path, None).await {
