@@ -10,7 +10,7 @@
 use super::register::EntryHash;
 use crate::{Error, Result, Safe};
 use log::debug;
-use safe_network::url::{SafeContentType, SafeUrl, XorUrl};
+use safe_network::url::{ContentType, NativeUrl, Scope, XorUrl};
 use std::collections::BTreeSet;
 use xor_name::XorName;
 
@@ -19,8 +19,6 @@ pub type MultimapValue = Vec<u8>;
 pub type MultimapKeyValue = (MultimapKey, MultimapValue);
 pub type MultimapKeyValues = BTreeSet<(EntryHash, MultimapKeyValue)>;
 
-const MULTIMAP_REMOVED_MARK: &[u8] = b"";
-
 impl Safe {
     /// Create a Multimap on the network
     pub async fn multimap_create(
@@ -35,12 +33,17 @@ impl Safe {
             .store_register(name, type_tag, None, private)
             .await?;
 
-        let xorurl = SafeUrl::encode_register(
+        let scope = if private {
+            Scope::Private
+        } else {
+            Scope::Public
+        };
+        let xorurl = NativeUrl::encode_register(
             xorname,
             type_tag,
-            SafeContentType::Multimap,
+            scope,
+            ContentType::Multimap,
             self.xorurl_base,
-            private,
         )?;
 
         Ok(xorurl)
@@ -59,7 +62,7 @@ impl Safe {
         &self,
         url: &str,
         hash: EntryHash,
-    ) -> Result<Option<MultimapKeyValue>> {
+    ) -> Result<MultimapKeyValue> {
         debug!("Getting value by hash from Multimap at: {}", url);
         let (safeurl, _) = self.parse_and_resolve_url(url).await?;
 
@@ -67,10 +70,10 @@ impl Safe {
     }
 
     // Return the value (by a provided key) of a Multimap on
-    // the network without resolving the SafeUrl
+    // the network without resolving the NativeUrl
     pub(crate) async fn fetch_multimap_value_by_key(
         &self,
-        safeurl: &SafeUrl,
+        safeurl: &NativeUrl,
         key: &[u8],
     ) -> Result<MultimapKeyValues> {
         let entries = self.fetch_multimap_values(safeurl).await?;
@@ -94,38 +97,31 @@ impl Safe {
                 entry, err
             ))
         })?;
-
+        let entry_xorname = self
+            .safe_client
+            .store_public_blob(&serialised_entry, false)
+            .await?;
+        let entry_xorurl = NativeUrl::encode_blob(
+            entry_xorname,
+            Scope::Public,
+            ContentType::Raw,
+            self.xorurl_base,
+        )?;
+        let entry_ptr = NativeUrl::from_xorurl(&entry_xorurl)?;
         let safeurl = Safe::parse_url(url)?;
         let address = safeurl.register_address()?;
 
         self.safe_client
-            .write_to_register(address, serialised_entry, replace)
+            .write_to_register(address, entry_ptr, replace)
             .await
     }
 
-    /// Remove a key from a Multimap on the network
-    pub async fn multimap_remove(
-        &self,
-        url: &str,
-        to_remove: BTreeSet<EntryHash>,
-    ) -> Result<EntryHash> {
-        debug!("Removing from Multimap at {}: {:?}", url, to_remove);
-        let safeurl = Safe::parse_url(url)?;
-        let address = safeurl.register_address()?;
-        let hash = self
-            .safe_client
-            .write_to_register(address, MULTIMAP_REMOVED_MARK.to_vec(), to_remove)
-            .await?;
-
-        Ok(hash)
-    }
-
     // Crate's helper to return the value of a Multimap on
-    // the network without resolving the SafeUrl,
+    // the network without resolving the NativeUrl,
     // optionally filtering by hash and/or key.
     pub(crate) async fn fetch_multimap_values(
         &self,
-        safeurl: &SafeUrl,
+        safeurl: &NativeUrl,
     ) -> Result<MultimapKeyValues> {
         let entries = match self.fetch_register_entries(safeurl).await {
             Ok(data) => {
@@ -144,26 +140,23 @@ impl Safe {
 
         // We parse each entry in the Register as a 'MultimapKeyValue'
         let mut multimap_key_vals = MultimapKeyValues::new();
-        for (hash, entry) in entries.iter() {
-            if entry == MULTIMAP_REMOVED_MARK {
-                // this is a tombstone entry created to delete some old entries
-                continue;
-            }
-            let key_val = Self::decode_multimap_entry(entry)?;
+        for (hash, entry_ptr) in entries.iter() {
+            let entry = self.fetch_public_blob(entry_ptr, None).await?;
+            let key_val = Self::decode_multimap_entry(&entry)?;
             multimap_key_vals.insert((*hash, key_val));
         }
         Ok(multimap_key_vals)
     }
 
     // Crate's helper to return the value of a Multimap on
-    // the network without resolving the SafeUrl,
+    // the network without resolving the NativeUrl,
     // optionally filtering by hash and/or key.
     pub(crate) async fn fetch_multimap_value_by_hash(
         &self,
-        safeurl: &SafeUrl,
+        safeurl: &NativeUrl,
         hash: EntryHash,
-    ) -> Result<Option<MultimapKeyValue>> {
-        let entry = match self.fetch_register_entry(safeurl, hash).await {
+    ) -> Result<MultimapKeyValue> {
+        let entry_ptr = match self.fetch_register_entry(safeurl, hash).await {
             Ok(data) => {
                 debug!("Multimap retrieved...");
                 Ok(data)
@@ -178,13 +171,9 @@ impl Safe {
             Err(other) => Err(other),
         }?;
 
-        // We parse each entry in the Register as a 'MultimapKeyValue'
-        if entry == MULTIMAP_REMOVED_MARK {
-            Ok(None)
-        } else {
-            let key_val = Self::decode_multimap_entry(&entry)?;
-            Ok(Some(key_val))
-        }
+        let entry = self.fetch_public_blob(&entry_ptr, None).await?;
+        let key_val = Self::decode_multimap_entry(&entry)?;
+        Ok(key_val)
     }
 
     fn decode_multimap_entry(entry: &[u8]) -> Result<MultimapKeyValue> {
@@ -308,75 +297,17 @@ mod tests {
             .multimap_insert(&xorurl_priv, key_val2.clone(), BTreeSet::new())
             .await?;
 
-        let received_data = retry_loop_for_pattern!(safe.multimap_get_by_hash(&xorurl, hash), Ok(v) if v.is_some())?;
-        let received_data_priv = retry_loop_for_pattern!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv), Ok(v) if v.is_some())?;
+        let received_data = retry_loop!(safe.multimap_get_by_hash(&xorurl, hash));
+        let received_data_priv = retry_loop!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv));
 
-        assert_eq!(received_data, Some(key_val.clone()));
-        assert_eq!(received_data_priv, Some(key_val));
+        assert_eq!(received_data, key_val.clone());
+        assert_eq!(received_data_priv, key_val);
 
-        let received_data = retry_loop_for_pattern!(safe.multimap_get_by_hash(&xorurl, hash2), Ok(v) if v.is_some())?;
-        let received_data_priv = retry_loop_for_pattern!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv2), Ok(v) if v.is_some())?;
+        let received_data = retry_loop!(safe.multimap_get_by_hash(&xorurl, hash2));
+        let received_data_priv = retry_loop!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv2));
 
-        assert_eq!(received_data, Some(key_val2.clone()));
-        assert_eq!(received_data_priv, Some(key_val2));
-
-        Ok(())
-    }
-
-    #[tokio::test]
-    async fn test_multimap_remove() -> Result<()> {
-        let safe = new_safe_instance().await?;
-        let key = b"key".to_vec();
-        let val = b"value".to_vec();
-        let key_val = (key.clone(), val.clone());
-
-        let xorurl = safe.multimap_create(None, 25_000, false).await?;
-        let xorurl_priv = safe.multimap_create(None, 25_000, true).await?;
-
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl, &key));
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl_priv, &key));
-
-        let hash = safe
-            .multimap_insert(&xorurl, key_val.clone(), BTreeSet::new())
-            .await?;
-        let hash_priv = safe
-            .multimap_insert(&xorurl_priv, key_val, BTreeSet::new())
-            .await?;
-
-        let received_data = retry_loop_for_pattern!(safe.multimap_get_by_key(&xorurl, &key), Ok(v) if !v.is_empty())?;
-        let received_data_priv = retry_loop_for_pattern!(safe.multimap_get_by_key(&xorurl_priv, &key), Ok(v) if !v.is_empty())?;
-
-        assert_eq!(received_data.len(), 1);
-        let (read_hash, read_key_val) = received_data.into_iter().next().unwrap();
-        assert_eq!(
-            (read_hash, read_key_val),
-            (hash, (key.clone(), val.clone()))
-        );
-
-        let hashes_to_remove = vec![hash].into_iter().collect();
-        let removed_mark_hash = safe.multimap_remove(&xorurl, hashes_to_remove).await?;
-        assert_ne!(removed_mark_hash, hash);
-
-        assert_eq!(
-            retry_loop_for_pattern!(safe.multimap_get_by_key(&xorurl, &key),
-                                    Ok(entries) if entries.is_empty())?,
-            Default::default()
-        );
-
-        assert_eq!(received_data_priv.len(), 1);
-        let (read_hash, read_key_val) = received_data_priv.into_iter().next().unwrap();
-        assert_eq!((read_hash, read_key_val), (hash_priv, (key.clone(), val)));
-
-        let hashes_to_remove = vec![hash_priv].into_iter().collect();
-        let removed_mark_hash = safe.multimap_remove(&xorurl_priv, hashes_to_remove).await?;
-        assert_ne!(removed_mark_hash, hash_priv);
-
-        assert_eq!(
-            retry_loop_for_pattern!(
-                safe.multimap_get_by_key(&xorurl_priv, &key),
-                Ok(entries) if entries.is_empty())?,
-            Default::default()
-        );
+        assert_eq!(received_data, key_val2.clone());
+        assert_eq!(received_data_priv, key_val2);
 
         Ok(())
     }
