@@ -10,16 +10,17 @@
 mod nrs_map;
 
 pub use nrs_map::{DefaultRdf, NrsMap};
+pub use safe_network::url::VersionHash;
 
 use crate::{
     app::{
         consts::{CONTENT_ADDED_SIGN, CONTENT_DELETED_SIGN},
         Safe,
     },
-    Error, Result, SafeContentType, SafeUrl, XorUrl,
+    Error, NativeUrl, Result, XorUrl,
 };
 use log::{debug, info, warn};
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, BTreeSet};
 
 // Type tag to use for the NrsMapContainer stored on Register
 pub(crate) const NRS_MAP_TYPE_TAG: u64 = 1_500;
@@ -30,19 +31,19 @@ const ERROR_MSG_NO_NRS_MAP_FOUND: &str = "No NRS Map found at this address";
 pub type ProcessedEntries = BTreeMap<String, (String, String)>;
 
 impl Safe {
-    pub fn parse_url(url: &str) -> Result<SafeUrl> {
-        let safe_url = SafeUrl::from_url(&sanitised_url(url))?;
+    pub fn parse_url(url: &str) -> Result<NativeUrl> {
+        let safe_url = NativeUrl::from_url(&sanitised_url(url))?;
         Ok(safe_url)
     }
 
-    // Parses a safe:// URL and returns all the info in a SafeUrl instance.
-    // It also returns a second SafeUrl if the URL was resolved from an NRS-URL,
-    // this second SafeUrl instance contains the information of the parsed NRS-URL.
+    // Parses a safe:// URL and returns all the info in a NativeUrl instance.
+    // It also returns a second NativeUrl if the URL was resolved from an NRS-URL,
+    // this second NativeUrl instance contains the information of the parsed NRS-URL.
     // *Note* this is not part of the public API, but an internal helper function used by API impl.
     pub(crate) async fn parse_and_resolve_url(
         &self,
         url: &str,
-    ) -> Result<(SafeUrl, Option<SafeUrl>)> {
+    ) -> Result<(NativeUrl, Option<NativeUrl>)> {
         let safe_url = Safe::parse_url(url)?;
         let orig_path = safe_url.path_decoded()?;
 
@@ -61,8 +62,8 @@ impl Safe {
             .pop()
             .ok_or_else(|| Error::ContentNotFound(format!("Failed to resolve {}", url)))?;
 
-        // Set the original path so we return the SafeUrl with it
-        let mut safe_url = SafeUrl::from_url(&safe_data.xorurl())?;
+        // Set the original path so we return the NativeUrl with it
+        let mut safe_url = NativeUrl::from_url(&safe_data.xorurl())?;
         safe_url.set_path(&orig_path);
 
         // If there is still one item in the chain, the first item is the NRS Map Container
@@ -70,7 +71,7 @@ impl Safe {
         if resolution_chain.is_empty() {
             Ok((safe_url, None))
         } else {
-            let nrsmap_xorul_encoder = SafeUrl::from_url(&resolution_chain[0].resolved_from())?;
+            let nrsmap_xorul_encoder = NativeUrl::from_url(&resolution_chain[0].resolved_from())?;
             Ok((safe_url, Some(nrsmap_xorul_encoder)))
         }
     }
@@ -82,7 +83,7 @@ impl Safe {
         default: bool,
         hard_link: bool,
         dry_run: bool,
-    ) -> Result<(u64, XorUrl, ProcessedEntries, NrsMap)> {
+    ) -> Result<(VersionHash, XorUrl, ProcessedEntries, NrsMap)> {
         info!("Adding to NRS map...");
         // GET current NRS map from name's TLD
         let (safe_url, _) = validate_nrs_name(name)?;
@@ -93,22 +94,25 @@ impl Safe {
         let link = nrs_map.update(name, link, default, hard_link)?;
         let mut processed_entries = ProcessedEntries::new();
         processed_entries.insert(name.to_string(), (CONTENT_ADDED_SIGN.to_string(), link));
-
         debug!("The new NRS Map: {:?}", nrs_map);
-        if !dry_run {
-            // Append new version of the NrsMap in the Public Sequence (NRS Map Container)
-            let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
-            self.safe_client
-                .append_to_sequence(
-                    nrs_map_xorurl.as_bytes(),
-                    safe_url.xorname(),
-                    safe_url.type_tag(),
-                    false,
-                )
-                .await?;
+
+        if dry_run {
+            return Err(Error::NotImplementedError(
+                "No dry run for nrs_map_container_add. Version info cannot be determined. (Register operations need this functionality implemented first.)".to_string(),
+            ));
         }
 
-        Ok((version + 1, xorurl, processed_entries, nrs_map))
+        let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
+        let mut old_values = BTreeSet::new();
+        old_values.insert(version.entry_hash());
+        let entry = (
+            name.as_bytes().to_owned(),
+            nrs_map_xorurl.as_bytes().to_owned(),
+        );
+        let entry_hash = &self.multimap_insert(&xorurl, entry, old_values).await?;
+        let new_version: VersionHash = entry_hash.into();
+
+        Ok((new_version, xorurl, processed_entries, nrs_map))
     }
 
     /// # Create a NrsMapContainer.
@@ -120,9 +124,8 @@ impl Safe {
     /// # use rand::{thread_rng, Rng};
     /// # use sn_api::Safe;
     /// # let mut safe = Safe::default();
-    /// # let rt = tokio::runtime::Runtime::new().unwrap();
-    /// # rt.block_on(async {
-    /// #   safe.connect(None, None, None).await.unwrap();
+    /// # async_std::task::block_on(async {
+    /// #   safe.connect("", Some("fake-credentials")).await.unwrap();
     ///     let rand_string: String = thread_rng().sample_iter(&Alphanumeric).take(15).collect();
     ///     let file_xorurl = safe.files_store_public_blob(&vec![], None, false).await.unwrap();
     ///     let (xorurl, _processed_entries, nrs_map_container) = safe.nrs_map_container_create(&rand_string, &file_xorurl, true, false, false).await.unwrap();
@@ -140,57 +143,50 @@ impl Safe {
         info!("Creating an NRS map");
         let (_, nrs_url) = validate_nrs_name(name)?;
         if self.nrs_map_container_get(&nrs_url).await.is_ok() {
-            Err(Error::ContentError(
+            return Err(Error::ContentError(
                 "NRS name already exists. Please use 'nrs add' command to add sub names to it"
                     .to_string(),
-            ))
-        } else {
-            let mut nrs_map = NrsMap::default();
-            let link = nrs_map.update(name, link, default, hard_link)?;
-            let mut processed_entries = ProcessedEntries::new();
-            processed_entries.insert(name.to_string(), (CONTENT_ADDED_SIGN.to_string(), link));
-
-            debug!("The new NRS Map: {:?}", nrs_map);
-            if dry_run {
-                Ok(("".to_string(), processed_entries, nrs_map))
-            } else {
-                let nrs_xorname = SafeUrl::from_nrsurl(&nrs_url)?.xorname();
-                debug!("XorName for \"{:?}\" is \"{:?}\"", &nrs_url, &nrs_xorname);
-
-                // Store the serialised NrsMap in a Public Blob
-                let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
-
-                // Store the NrsMapContainer in a Public Sequence, putting the
-                // serialised NrsMap XOR-URL as the first entry value
-                let xorname = self
-                    .safe_client
-                    .store_sequence(
-                        nrs_map_xorurl.as_bytes(),
-                        Some(nrs_xorname),
-                        NRS_MAP_TYPE_TAG,
-                        None,
-                        false,
-                    )
-                    .await?;
-
-                let xorurl = SafeUrl::encode_sequence_data(
-                    xorname,
-                    NRS_MAP_TYPE_TAG,
-                    SafeContentType::NrsMapContainer,
-                    self.xorurl_base,
-                    false,
-                )?;
-
-                Ok((xorurl, processed_entries, nrs_map))
-            }
+            ));
         }
+
+        let mut nrs_map = NrsMap::default();
+        let link = nrs_map.update(name, link, default, hard_link)?;
+        let mut processed_entries = ProcessedEntries::new();
+        processed_entries.insert(name.to_string(), (CONTENT_ADDED_SIGN.to_string(), link));
+        debug!("The new NRS Map: {:?}", nrs_map);
+
+        if dry_run {
+            return Ok(("".to_string(), processed_entries, nrs_map));
+        }
+
+        let nrs_xorname = NativeUrl::from_nrsurl(&nrs_url)?.xorname();
+        debug!("XorName for \"{:?}\" is \"{:?}\"", &nrs_url, &nrs_xorname);
+
+        // Store the serialised NrsMap in a Public Blob
+        let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
+
+        // Create a new multimap
+        let xorurl = self
+            .multimap_create(Some(nrs_xorname), NRS_MAP_TYPE_TAG, false)
+            .await?;
+
+        // Store the NRS map in the multimap
+        let entry = (
+            name.as_bytes().to_owned(),
+            nrs_map_xorurl.as_bytes().to_owned(),
+        );
+        let _ = self
+            .multimap_insert(&xorurl, entry, BTreeSet::new())
+            .await?;
+
+        Ok((xorurl, processed_entries, nrs_map))
     }
 
     pub async fn nrs_map_container_remove(
         &self,
         name: &str,
         dry_run: bool,
-    ) -> Result<(u64, XorUrl, ProcessedEntries, NrsMap)> {
+    ) -> Result<(VersionHash, XorUrl, ProcessedEntries, NrsMap)> {
         info!("Removing from NRS map...");
         // GET current NRS map from &name TLD
         let (safe_url, _) = validate_nrs_name(name)?;
@@ -205,35 +201,41 @@ impl Safe {
             (CONTENT_DELETED_SIGN.to_string(), removed_link),
         );
 
-        debug!("The new NRS Map: {:?}", nrs_map);
-        if !dry_run {
-            // Append new version of the NrsMap in the Public Sequence (NRS Map Container)
-            let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
-            self.safe_client
-                .append_to_sequence(
-                    nrs_map_xorurl.as_bytes(),
-                    safe_url.xorname(),
-                    safe_url.type_tag(),
-                    false,
-                )
-                .await?;
+        if dry_run {
+            return Err(Error::NotImplementedError(
+                "No dry run for nrs_map_container_remove. Version info cannot be determined. (Register operations need this functionality implemented first.)".to_string(),
+            ));
         }
 
-        Ok((version + 1, xorurl, processed_entries, nrs_map))
+        debug!("Removing from multimap");
+        let mut old_values = BTreeSet::new();
+        old_values.insert(version.entry_hash());
+
+        // TODO use remove
+        // self.multimap_remove(&xorurl, old_values).await?;
+        // tmp retro-compatible workaround with insert
+        let nrs_map_xorurl = self.store_nrs_map(&nrs_map).await?;
+        let entry = (
+            name.as_bytes().to_owned(),
+            nrs_map_xorurl.as_bytes().to_owned(),
+        );
+        let entry_hash = &self.multimap_insert(&xorurl, entry, old_values).await?;
+        let new_version: VersionHash = entry_hash.into();
+
+        Ok((new_version, xorurl, processed_entries, nrs_map))
     }
 
     /// # Fetch an existing NrsMapContainer.
     ///
     /// ## Example
     ///
-    /// ```rust,no_run
+    /// ```rust_no_run
     /// # use sn_api::Safe;
     /// # use rand::distributions::Alphanumeric;
     /// # use rand::{thread_rng, Rng};
     /// # let mut safe = Safe::default();
-    /// # let rt = tokio::runtime::Runtime::new().unwrap();
-    /// # rt.block_on(async {
-    /// #   safe.connect(None, None, None).await.unwrap();
+    /// # async_std::task::block_on(async {
+    /// #   safe.connect("", Some("fake-credentials")).await.unwrap();
     ///     let rand_string: String = thread_rng().sample_iter(&Alphanumeric).take(15).collect();
     ///     let file_xorurl = safe.files_store_public_blob(&vec![], Some("text/plain"), false).await.unwrap();
     ///     let (xorurl, _processed_entries, _nrs_map) = safe.nrs_map_container_create(&rand_string, &file_xorurl, true, false, false).await.unwrap();
@@ -242,83 +244,67 @@ impl Safe {
     ///     assert_eq!(nrs_map_container.get_default_link().unwrap(), file_xorurl);
     /// # });
     /// ```
-    pub async fn nrs_map_container_get(&self, url: &str) -> Result<(u64, NrsMap)> {
+    pub async fn nrs_map_container_get(&self, url: &str) -> Result<(VersionHash, NrsMap)> {
         debug!("Getting latest resolvable map container from: {:?}", url);
         let safe_url = Safe::parse_url(url)?;
 
-        // Check if the URL specified a specific version of the content or simply the latest available
-        let data = match safe_url.content_version() {
-            None => {
-                self.safe_client
-                    .sequence_get_last_entry(safe_url.xorname(), NRS_MAP_TYPE_TAG, false)
-                    .await
-            }
-            Some(content_version) => {
-                let serialised_nrs_map = self
-                    .safe_client
-                    .sequence_get_entry(
-                        safe_url.xorname(),
-                        NRS_MAP_TYPE_TAG,
-                        content_version,
-                        false,
-                    )
-                    .await
-                    .map_err(|_| {
-                        Error::VersionNotFound(format!(
-                            "Version '{}' is invalid for NRS Map Container found at \"{}\"",
-                            content_version, url,
-                        ))
-                    })?;
+        // fetch latest entries and wrap errors
+        let entries = self
+            .fetch_multimap_values(&safe_url)
+            .await
+            .map_err(|e| match e {
+                Error::ContentNotFound(_) => {
+                    Error::ContentNotFound(ERROR_MSG_NO_NRS_MAP_FOUND.to_string())
+                }
+                Error::VersionNotFound(msg) => Error::VersionNotFound(msg),
+                err => Error::NetDataError(format!("Failed to get current version: {}", err)),
+            })?;
 
-                Ok((content_version, serialised_nrs_map))
-            }
-        };
-
-        match data {
-            Ok((version, nrs_map_xorurl_bytes)) => {
-                // We first parse the NrsMap XOR-URL from the Sequence
-                let url = String::from_utf8(nrs_map_xorurl_bytes).map_err(|err| {
-                    Error::ContentError(format!(
-                        "Couldn't parse the NrsMap link stored in the NrsMapContainer: {:?}",
-                        err
-                    ))
-                })?;
-                debug!("Deserialised NrsMap XOR-URL: {}", url);
-                let nrs_map_xorurl = SafeUrl::from_url(&url)?;
-
-                // Using the NrsMap XOR-URL we can now fetch the NrsMap and deserialise it
-                let serialised_nrs_map = self.fetch_public_blob(&nrs_map_xorurl, None).await?;
-
-                debug!("Nrs map v{} retrieved: {:?} ", version, &serialised_nrs_map);
-                let nrs_map =
-                    serde_json::from_str(&String::from_utf8_lossy(serialised_nrs_map.as_slice()))
-                        .map_err(|err| {
-                        Error::ContentError(format!(
-                            "Couldn't deserialise the NrsMap stored in the NrsContainer: {:?}",
-                            err
-                        ))
-                    })?;
-
-                Ok((version, nrs_map))
-            }
-            Err(Error::EmptyContent(_)) => {
-                warn!("Nrs container found at {:?} was empty", &url);
-                Ok((0, NrsMap::default()))
-            }
-            Err(Error::ContentNotFound(_)) => Err(Error::ContentNotFound(
-                ERROR_MSG_NO_NRS_MAP_FOUND.to_string(),
-            )),
-            Err(Error::VersionNotFound(msg)) => Err(Error::VersionNotFound(msg)),
-            Err(err) => Err(Error::NetDataError(format!(
-                "Failed to get current version: {}",
-                err
-            ))),
+        // take the 1st entry (TODO Multiple entries)
+        if entries.len() > 1 {
+            return Err(Error::MultimapFork("Multiple NRS map entries not managed, this happends when 2 clients write concurrently to a NRS map".to_string()));
         }
+        let first_entry = entries.iter().next();
+        let (version, nrs_map_xorurl_bytes);
+        if let Some((v, (_name, m))) = first_entry {
+            version = v.into();
+            nrs_map_xorurl_bytes = m.to_owned();
+        } else {
+            warn!(
+                "NRS map Register found at XOR name \"{:?}\" was empty",
+                safe_url.xorname()
+            );
+            return Ok((VersionHash::default(), NrsMap::default()));
+        }
+
+        // get the xorurl from nrs container
+        let url = String::from_utf8(nrs_map_xorurl_bytes.to_owned()).map_err(|err| {
+            Error::ContentError(format!(
+                "Couldn't parse the NrsMap link stored in the NrsMapContainer: {:?}",
+                err
+            ))
+        })?;
+        debug!("Deserialised NrsMap XOR-URL: {}", url);
+        let nrs_map_xorurl = NativeUrl::from_url(&url)?;
+
+        // Using the NrsMap XOR-URL we can now fetch the NrsMap and deserialise it
+        let serialised_nrs_map = self.fetch_public_blob(&nrs_map_xorurl, None).await?;
+
+        debug!("Nrs map v{} retrieved: {:?} ", version, &serialised_nrs_map);
+        let nrs_map = serde_json::from_str(&String::from_utf8_lossy(serialised_nrs_map.as_slice()))
+            .map_err(|err| {
+                Error::ContentError(format!(
+                    "Couldn't deserialise the NrsMap stored in the NrsContainer: {:?}",
+                    err
+                ))
+            })?;
+
+        Ok((version, nrs_map))
     }
 
     // Private helper to serialise an NrsMap and store it in a Public Blob
     async fn store_nrs_map(&self, nrs_map: &NrsMap) -> Result<String> {
-        // The NrsMapContainer is a Sequence where each NRS Map version is
+        // The NrsMapContainer is a Register where each NRS Map version is
         // an entry containing the XOR-URL of the Blob that contains the serialised NrsMap.
         // TODO: use RDF format
         let serialised_nrs_map = serde_json::to_string(nrs_map).map_err(|err| {
@@ -336,7 +322,7 @@ impl Safe {
     }
 }
 
-fn validate_nrs_name(name: &str) -> Result<(SafeUrl, String)> {
+fn validate_nrs_name(name: &str) -> Result<(NativeUrl, String)> {
     // validate no slashes in name.
     if name.find('/').is_some() {
         let msg = "The NRS name/subname cannot contain a slash".to_string();
@@ -379,20 +365,18 @@ mod tests {
 
         let nrs_xorname = Safe::parse_url(&site_name)?.xorname();
 
-        let (xor_url, _, nrs_map) = safe
-            .nrs_map_container_create(
-                &site_name,
-                "safe://linked-from-site_name?v=0",
-                true,
-                false,
-                false,
-            )
-            .await?;
+        let (xor_url, _, nrs_map) = retry_loop!(safe.nrs_map_container_create(
+            &site_name,
+            &format!("safe://linked-from-site_name?v={}", VersionHash::default()),
+            true,
+            false,
+            false
+        ));
 
         assert_eq!(nrs_map.sub_names_map.len(), 0);
         assert_eq!(
             nrs_map.get_default_link()?,
-            "safe://linked-from-site_name?v=0"
+            format!("safe://linked-from-site_name?v={}", VersionHash::default())
         );
 
         if let DefaultRdf::OtherRdf(def_data) = &nrs_map.default {
@@ -400,12 +384,15 @@ mod tests {
                 .get(PREDICATE_LINK)
                 .ok_or_else(|| anyhow!("Entry not found with key '{}'", PREDICATE_LINK))?;
 
-            assert_eq!(*link, "safe://linked-from-site_name?v=0".to_string());
+            assert_eq!(
+                *link,
+                format!("safe://linked-from-site_name?v={}", VersionHash::default())
+            );
             assert_eq!(
                 nrs_map.get_default()?,
                 &DefaultRdf::OtherRdf(def_data.clone())
             );
-            let decoder = SafeUrl::from_url(&xor_url)?;
+            let decoder = NativeUrl::from_url(&xor_url)?;
             assert_eq!(nrs_xorname, decoder.xorname());
             Ok(())
         } else {
@@ -422,21 +409,32 @@ mod tests {
         let (link, _, _) = safe
             .files_container_create(None, None, true, true, false)
             .await?;
-        let link_v0 = format!("{}?v=0", link);
+        let (version0, _) = retry_loop!(safe.files_container_get(&link));
+        let link_v0 = format!("{}?v={}", link, version0);
+
+        let (xorurl, _, nrs_map) = retry_loop!(safe.nrs_map_container_create(
+            &format!("b.{}", site_name),
+            &link_v0,
+            true,
+            false,
+            false
+        ));
 
-        let (xorurl, _, nrs_map) = safe
-            .nrs_map_container_create(&format!("b.{}", site_name), &link_v0, true, false, false)
-            .await?;
         assert_eq!(nrs_map.sub_names_map.len(), 1);
         assert_eq!(nrs_map.get_default_link()?, link_v0);
         let _ = retry_loop!(safe.fetch(&xorurl, None));
 
         // add subname and set it as the new default too
-        let link_v1 = format!("{}?v=1", link);
-        let (version, _, _, updated_nrs_map) = safe
-            .nrs_map_container_add(&format!("a.b.{}", site_name), &link_v1, true, false, false)
-            .await?;
-        assert_eq!(version, 1);
+        let link_v1 = format!("{}?v={}", link, VersionHash::default());
+        let (version, _, _, updated_nrs_map) = retry_loop!(safe.nrs_map_container_add(
+            &format!("a.b.{}", site_name),
+            &link_v1,
+            true,
+            false,
+            false
+        ));
+
+        assert_ne!(version, version0);
         assert_eq!(updated_nrs_map.sub_names_map.len(), 1);
         assert_eq!(updated_nrs_map.get_default_link()?, link_v1);
 
@@ -452,19 +450,25 @@ mod tests {
         let (link, _, _) = safe
             .files_container_create(None, None, true, true, false)
             .await?;
-        let link_v0 = format!("{}?v=0", link);
+        let (version0, _) = retry_loop!(safe.files_container_get(&link));
+        let link_v0 = format!("{}?v={}", link, version0);
 
-        let (xorurl, _, _) = safe
-            .nrs_map_container_create(&format!("b.{}", site_name), &link_v0, true, false, false)
-            .await?;
+        let (xorurl, _, _) = retry_loop!(safe.nrs_map_container_create(
+            &format!("b.{}", site_name),
+            &link_v0,
+            true,
+            false,
+            false
+        ));
 
         let _ = retry_loop!(safe.fetch(&xorurl, None));
 
-        let versioned_sitename = format!("a.b.{}?v=6", site_name);
+        let dummy_version = "hqt1zg7dwci3ze7dfqp48e3muqt4gkh5wqt1zg7dwci3ze7dfqp4y";
+        let versioned_sitename = format!("a.b.{}?v={}", site_name, dummy_version);
         match safe
             .nrs_map_container_add(
                 &versioned_sitename,
-                "safe://linked-from-a_b_site_name?v=0",
+                &format!("safe://linked-from-a_b_site_name?v={}", version0),
                 true,
                 false,
                 false,
@@ -519,27 +523,37 @@ mod tests {
         let (link, _, _) = safe
             .files_container_create(None, None, true, true, false)
             .await?;
-        let link_v0 = format!("{}?v=0", link);
+        let (version0, _) = retry_loop!(safe.files_container_get(&link));
+        let link_v0 = format!("{}?v={}", link, version0);
 
-        let (xorurl, _, nrs_map) = safe
-            .nrs_map_container_create(&format!("a.b.{}", site_name), &link_v0, true, false, false)
-            .await?;
+        let (xorurl, _, nrs_map) = retry_loop!(safe.nrs_map_container_create(
+            &format!("a.b.{}", site_name),
+            &link_v0,
+            true,
+            false,
+            false,
+        ));
         assert_eq!(nrs_map.sub_names_map.len(), 1);
         let _ = retry_loop!(safe.fetch(&xorurl, None));
 
-        let link_v1 = format!("{}?v=1", link);
-        let _ = safe
-            .nrs_map_container_add(&format!("a2.b.{}", site_name), &link_v1, true, false, false)
-            .await?;
+        let dummy_version = "hqt1zg7dwci3ze7dfqp48e3muqt4gkh5wqt1zg7dwci3ze7dfqp4y";
+        let link_v1 = format!("{}?v={}", link, dummy_version);
+        let (version1, _, _, _) = retry_loop!(safe.nrs_map_container_add(
+            &format!("a2.b.{}", site_name),
+            &link_v1,
+            true,
+            false,
+            false
+        ));
 
-        let _ = retry_loop_for_pattern!(safe.nrs_map_container_get(&xorurl), Ok((version, _)) if *version == 1)?;
+        let _ = retry_loop_for_pattern!(safe.nrs_map_container_get(&xorurl), Ok((version, _)) if *version == version1)?;
 
         // remove subname
-        let (version, _, _, updated_nrs_map) = safe
-            .nrs_map_container_remove(&format!("a.b.{}", site_name), false)
-            .await?;
+        let (version, _, _, updated_nrs_map) =
+            retry_loop!(safe.nrs_map_container_remove(&format!("a.b.{}", site_name), false));
 
-        assert_eq!(version, 2);
+        assert_ne!(version, version0);
+        assert_ne!(version, version1);
         assert_eq!(updated_nrs_map.sub_names_map.len(), 1);
         assert_eq!(updated_nrs_map.get_default_link()?, link_v1);
 
@@ -555,19 +569,23 @@ mod tests {
         let (link, _, _) = safe
             .files_container_create(None, None, true, true, false)
             .await?;
-        let link_v0 = format!("{}?v=0", link);
+        let (version0, _) = retry_loop!(safe.files_container_get(&link));
+        let link_v0 = format!("{}?v={}", link, version0);
 
-        let (xorurl, _, nrs_map) = safe
-            .nrs_map_container_create(&format!("a.b.{}", site_name), &link_v0, true, false, false)
-            .await?;
+        let (xorurl, _, nrs_map) = retry_loop!(safe.nrs_map_container_create(
+            &format!("a.b.{}", site_name),
+            &link_v0,
+            true,
+            false,
+            false,
+        ));
         assert_eq!(nrs_map.sub_names_map.len(), 1);
         let _ = retry_loop!(safe.fetch(&xorurl, None));
 
         // remove subname
-        let (version, _, _, updated_nrs_map) = safe
-            .nrs_map_container_remove(&format!("a.b.{}", site_name), false)
-            .await?;
-        assert_eq!(version, 1);
+        let (_version, _, _, updated_nrs_map) =
+            retry_loop!(safe.nrs_map_container_remove(&format!("a.b.{}", site_name), false));
+        // assert_eq!(version, 1);
         assert_eq!(updated_nrs_map.sub_names_map.len(), 0);
         match updated_nrs_map.get_default_link() {
             Ok(link) => Err(anyhow!("Unexpectedly retrieved a default link: {}", link)),
@@ -592,25 +610,24 @@ mod tests {
         let (link, _, _) = safe
             .files_container_create(None, None, true, true, false)
             .await?;
-        let link_v0 = format!("{}?v=0", link);
+        let (version0, _) = retry_loop!(safe.files_container_get(&link));
+        let link_v0 = format!("{}?v={}", link, version0);
 
-        let (xorurl, _, nrs_map) = safe
-            .nrs_map_container_create(
-                &format!("a.b.{}", site_name),
-                &link_v0,
-                true,
-                true, // this sets the default to be a hard-link
-                false,
-            )
-            .await?;
+        let (xorurl, _, nrs_map) = retry_loop!(safe.nrs_map_container_create(
+            &format!("a.b.{}", site_name),
+            &link_v0,
+            true,
+            true, // this sets the default to be a hard-link
+            false,
+        ));
         assert_eq!(nrs_map.sub_names_map.len(), 1);
         let _ = retry_loop!(safe.fetch(&xorurl, None));
 
         // remove subname
-        let (version, _, _, updated_nrs_map) = safe
-            .nrs_map_container_remove(&format!("a.b.{}", site_name), false)
-            .await?;
-        assert_eq!(version, 1);
+        let (version, _, _, updated_nrs_map) =
+            retry_loop!(safe.nrs_map_container_remove(&format!("a.b.{}", site_name), false));
+
+        assert_ne!(version, version0);
         assert_eq!(updated_nrs_map.sub_names_map.len(), 0);
         assert_eq!(updated_nrs_map.get_default_link()?, link_v0);
         Ok(())
