@@ -11,7 +11,7 @@ pub use safe_network::types::register::{Entry, EntryHash};
 
 use crate::{Error, Result, Safe};
 use log::debug;
-use safe_network::url::{SafeContentType, SafeUrl, XorUrl};
+use safe_network::url::{ContentType, NativeUrl, Scope, XorUrl};
 use std::collections::BTreeSet;
 use xor_name::XorName;
 
@@ -28,12 +28,17 @@ impl Safe {
             .store_register(name, type_tag, None, private)
             .await?;
 
-        let xorurl = SafeUrl::encode_register(
+        let scope = if private {
+            Scope::Private
+        } else {
+            Scope::Public
+        };
+        let xorurl = NativeUrl::encode_register(
             xorname,
             type_tag,
-            SafeContentType::Raw,
+            scope,
+            ContentType::Raw,
             self.xorurl_base,
-            private,
         )?;
 
         Ok(xorurl)
@@ -55,39 +60,52 @@ impl Safe {
         self.fetch_register_entry(&safeurl, hash).await
     }
 
-    /// Fetch a Register from a SafeUrl without performing any type of URL resolution
+    /// Fetch a Register from a NativeUrl without performing any type of URL resolution
+    /// Supports version hashes:
+    /// e.g. safe://mysafeurl?v=ce56a3504c8f27bfeb13bdf9051c2e91409230ea
     pub(crate) async fn fetch_register_entries(
         &self,
-        safeurl: &SafeUrl,
+        safeurl: &NativeUrl,
     ) -> Result<BTreeSet<(EntryHash, Entry)>> {
-        // TODO: allow to specify the hash with the SafeUrl as well: safeurl.content_hash(),
-        // e.g. safe://mysafeurl#ce56a3504c8f27bfeb13bdf9051c2e91409230ea
+        let result = match safeurl.content_version() {
+            Some(v) => {
+                // take entry with version hash
+                let hash = v.entry_hash();
+                self.fetch_register_entry(safeurl, hash)
+                    .await
+                    .map(|entry| vec![(hash, entry)].into_iter().collect())
+            }
+            None => {
+                // then take latest entry
+                let address = safeurl.register_address()?;
+                self.safe_client.read_register(address).await
+            }
+        };
 
-        let address = safeurl.register_address()?;
-
-        match self.safe_client.read_register(address).await {
+        match result {
             Ok(data) => {
-                debug!("Register retrieved...");
+                debug!("Register retrieved from {}...", safeurl);
                 Ok(data)
             }
             Err(Error::EmptyContent(_)) => Err(Error::EmptyContent(format!(
                 "Register found at \"{}\" was empty",
                 safeurl
             ))),
-            Err(Error::ContentNotFound(_)) => Err(Error::ContentNotFound(
-                "No Register found at this address".to_string(),
-            )),
-            other => other,
+            Err(Error::ContentNotFound(_)) => Err(Error::ContentNotFound(format!(
+                "No Register found at \"{}\"",
+                safeurl
+            ))),
+            other_err => other_err,
         }
     }
 
-    /// Fetch a Register from a SafeUrl without performing any type of URL resolution
+    /// Fetch a Register from a NativeUrl without performing any type of URL resolution
     pub(crate) async fn fetch_register_entry(
         &self,
-        safeurl: &SafeUrl,
+        safeurl: &NativeUrl,
         hash: EntryHash,
     ) -> Result<Entry> {
-        // TODO: allow to specify the hash with the SafeUrl as well: safeurl.content_hash(),
+        // TODO: allow to specify the hash with the NativeUrl as well: safeurl.content_hash(),
         // e.g. safe://mysafeurl#ce56a3504c8f27bfeb13bdf9051c2e91409230ea
         let address = safeurl.register_address()?;
 
@@ -99,7 +117,7 @@ impl Safe {
     pub async fn write_to_register(
         &self,
         url: &str,
-        data: Vec<u8>,
+        entry: Entry,
         parents: BTreeSet<EntryHash>,
     ) -> Result<EntryHash> {
         /*
@@ -117,16 +135,16 @@ impl Safe {
         let (safeurl, _) = self.parse_and_resolve_url(url).await?;
         let address = safeurl.register_address()?;
 
-        // write the data to the Register
+        // write the entry to the Register
         self.safe_client
-            .write_to_register(address, data, parents)
+            .write_to_register(address, entry, parents)
             .await
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{app::test_helpers::new_safe_instance, retry_loop};
+    use crate::{app::test_helpers::new_safe_instance, retry_loop, NativeUrl};
     use anyhow::Result;
 
     #[tokio::test]
@@ -142,19 +160,19 @@ mod tests {
         assert!(received_data.is_empty());
         assert!(received_data_priv.is_empty());
 
-        let initial_data = b"initial data";
+        let initial_data = NativeUrl::from_url("safe://test")?;
         let hash = safe
-            .write_to_register(&xorurl, initial_data.to_vec(), Default::default())
+            .write_to_register(&xorurl, initial_data.clone(), Default::default())
             .await?;
         let hash_priv = safe
-            .write_to_register(&xorurl_priv, initial_data.to_vec(), Default::default())
+            .write_to_register(&xorurl_priv, initial_data.clone(), Default::default())
             .await?;
 
         let received_entry = retry_loop!(safe.register_read_entry(&xorurl, hash));
         let received_entry_priv = retry_loop!(safe.register_read_entry(&xorurl_priv, hash_priv));
 
-        assert_eq!(received_entry, initial_data.to_vec());
-        assert_eq!(received_entry_priv, initial_data.to_vec());
+        assert_eq!(received_entry, initial_data.clone());
+        assert_eq!(received_entry_priv, initial_data);
 
         Ok(())
     }
