@@ -116,7 +116,7 @@ fn get_bootstrap_contacts() -> Result<HashSet<SocketAddr>> {
 #[macro_export]
 macro_rules! retry_loop {
     ($n:literal, $async_func:expr) => {{
-        let mut retries = $n;
+        let mut retries: u64 = $n;
         loop {
             match $async_func.await {
                 Ok(val) => break val,
@@ -136,13 +136,23 @@ macro_rules! retry_loop {
 
 #[macro_export]
 macro_rules! retry_loop_for_pattern {
-    ($async_func:expr, $pattern:pat $(if $cond:expr)?) => {
+    ($n:literal, $async_func:expr, $pattern:pat $(if $cond:expr)?) => {{
+        let mut retries: u64 = $n;
         loop {
             let result = $async_func.await;
             match &result {
                 $pattern $(if $cond)? => break result,
-                Ok(_) | Err(_) => tokio::time::sleep(std::time::Duration::from_millis(300)).await,
+                Ok(_) | Err(_) if retries > 0 => {
+                    retries -= 1;
+                    tokio::time::sleep(std::time::Duration::from_millis(1000)).await;
+                },
+                Err(e) => anyhow::bail!("Failed after {} retries: {:?}", $n, e),
+                Ok(_) => anyhow::bail!("Failed to match pattern after {} retries", $n),
             }
         }
-    };
+    }};
+    // Defaults to 10 retries if n is not provided
+    ($async_func:expr, $pattern:pat $(if $cond:expr)?) => {{
+        retry_loop_for_pattern!(10, $async_func, $pattern $(if $cond)?)
+    }};
 }
